unit fcmain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Buttons, msEstTimer, OnGuard;

type
  TFCForm = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    InputFileEdit: TEdit;
    OutputFileEdit: TEdit;
    InputFileBrowse: TSpeedButton;
    OutputFileBrowse: TSpeedButton;
    StartButton: TButton;
    CancelButton: TButton;
    StatusBar: TStatusBar;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    msEstTimer1: TmsEstTimer;
    procedure InputFileBrowseClick(Sender: TObject);
    procedure OutputFileBrowseClick(Sender: TObject);
    procedure StartButtonClick(Sender: TObject);
    procedure msEstTimer1DisplayElapsedTime(Sender: TObject;
      TheTime: TDateTime);
    procedure msEstTimer1DisplayEstTime(Sender: TObject;
      TheTime: TDateTime);
    procedure msEstTimer1GetCurrentCount(Sender: TObject;
      var Count: Integer);
    procedure msEstTimer1GetTotalCount(Sender: TObject;
      var Count: Integer);
    procedure CancelButtonClick(Sender: TObject);
  private
    { Private declarations }
    procedure Verify;
    procedure CopyFile;
  protected
    { Protected declarations }
    CurrentByte: LongInt;
    TotalBytes: LongInt;
    Canceled: boolean;
  public
    { Public declarations }
  end;

var
  FCForm: TFCForm;

implementation

{$R *.DFM}

procedure TFCForm.InputFileBrowseClick(Sender: TObject);
begin
  if OpenDialog.Execute then
    InputFileEdit.Text:=OpenDialog.FileName;
end;

procedure TFCForm.OutputFileBrowseClick(Sender: TObject);
begin
  if SaveDialog.Execute then
    OutputFileEdit.Text:=SaveDialog.FileName;
end;

procedure TFCForm.Verify;
begin
  if InputFileEdit.Text='' then
    raise Exception.Create('Input file name is not specified');
  if OutputFileEdit.Text='' then
    raise Exception.Create('Output file name is not specified');
end;

procedure TFCForm.StartButtonClick(Sender: TObject);
begin
  Verify;
  if FileExists(OutputFileEdit.Text) and
    (MessageDlg('Output file exists. Overwrite?',
     mtWarning,[mbYes,mbNo],0)<>mrYes) then Exit;
  StartButton.Enabled:=false;
  CancelButton.Enabled:=true;
  msEstTimer1.Enabled:=true;
  try
    CopyFile;
    msEstTimer1.Enabled:=false;
    if Canceled then
      MessageDlg('Operation has been canceled',mtInformation,[mbOk],0)
    else
      MessageDlg('File has been copied',mtInformation,[mbOk],0);
  finally
    CancelButton.Enabled:=false;
    StartButton.Enabled:=true;
  end;
end;

procedure TFCForm.CopyFile;
const
  BufSize=$10; // use small buffer, to make it slow
var
  InFile, OutFile: file;
  Buf: pointer;
  BytesRead: Integer;
begin
  AssignFile(InFile,InputFileEdit.Text);
  Reset(InFile,1);
  try
    AssignFile(OutFile,OutputFileEdit.Text);
    Rewrite(OutFile,1);
    try
      GetMem(Buf,BufSize);
      try
        TotalBytes:=FileSize(InFile);
        CurrentByte:=0; Canceled:=false;
        while not EOF(InFile) and not Canceled do
        begin
          BlockRead(InFile,Buf^,BufSize,BytesRead);
          BlockWrite(OutFile,Buf^,BytesRead);
          CurrentByte:=CurrentByte+BytesRead;
          Application.ProcessMessages;
        end;
      finally
        FreeMem(Buf,BufSize);
      end;
    finally
      CloseFile(OutFile);
    end;
  finally
    CloseFile(InFile);
  end;
end;

procedure TFCForm.msEstTimer1DisplayElapsedTime(Sender: TObject;
  TheTime: TDateTime);
begin
  StatusBar.Panels[0].Text:='Elapsed Time: '+FormatDateTime('hh:mm:ss',TheTime);
end;

procedure TFCForm.msEstTimer1DisplayEstTime(Sender: TObject;
  TheTime: TDateTime);
begin
  StatusBar.Panels[1].Text:='Estimated Time: '+FormatDateTime('hh:mm:ss',TheTime);
end;

procedure TFCForm.msEstTimer1GetCurrentCount(Sender: TObject;
  var Count: Integer);
begin
  Count:=CurrentByte;
end;

procedure TFCForm.msEstTimer1GetTotalCount(Sender: TObject;
  var Count: Integer);
begin
  Count:=TotalBytes;
end;

procedure TFCForm.CancelButtonClick(Sender: TObject);
begin
  Canceled:=true;
end;

end.
