unit msEstTimer;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Consts, msetAbout, DsgnIntf;

const
  VersionStr='1.0';  

type
  TmsGetCountEvent=procedure(Sender: TObject; var Count: LongInt) of Object;
  TmsDisplayTimeEvent=procedure(Sender: TObject; TheTime: TDateTime) of Object;

  TmsEstTimer = class(TComponent)
  private
    { Private declarations }
    FInvWindow: HWND;
    FInterval: Cardinal;
    FEnabled: boolean;
    FOnGetCurrentCount: TmsGetCountEvent;
    FOnGetTotalCount: TmsGetCountEvent;
    FOnDisplayEstTime: TmsDisplayTimeEvent;
    FOnDisplayElapsedTime: TmsDisplayTimeEvent;
    function GetVersion: string;
    procedure SetVersion(const Value: string);
    procedure SetInterval(Value: Cardinal);
    procedure SetEnabled(Value: boolean);
    procedure UpdateTimer;
  protected
    { Protected declarations }
    FStartTime: TDateTime;
    procedure WndProc(var Msg: TMessage);
    procedure TimerOnTimer(Sender: TObject);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Interval: Cardinal read FInterval write SetInterval;
    property Enabled: boolean read FEnabled write SetEnabled;
    property Version: string read GetVersion write SetVersion;
    property OnGetCurrentCount: TmsGetCountEvent read FOnGetCurrentCount
      write FOnGetCurrentCount;
    property OnGetTotalCount: TmsGetCountEvent read FOnGetTotalCount
      write FOnGetTotalCount;
    property OnDisplayEstTime: TmsDisplayTimeEvent read FOnDisplayEstTime
      write FOnDisplayEstTime;
    property OnDisplayElapsedTime: TmsDisplayTimeEvent read FOnDisplayElapsedTime
      write FOnDisplayElapsedTime;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('ArgoSoft', [TmsEstTimer]);
  RegisterPropertyEditor(TypeInfo(string), TmsEstTimer, 'Version', TmsVersionProperty);
end;

constructor TmsEstTimer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FInterval:=1000;
  FEnabled:=false;
  FInvWindow:=AllocateHWnd(WndProc);
end;

destructor TmsEstTimer.Destroy;
begin
  FEnabled:=false;
  UpdateTimer;
  DeallocateHWnd(FInvWindow);
  inherited Destroy;
end;

function TmsEstTimer.GetVersion: string;
begin
  Result:=VersionStr;
end;

procedure TmsEstTimer.SetVersion(const Value: string);
begin

end;

procedure TmsEstTimer.WndProc(var Msg: TMessage);
begin
  with Msg do
    if Msg=WM_TIMER then
    try
      TimerOnTimer(Self);
    except
      Application.HandleException(Self);
    end
    else
      Result:=DefWindowProc(FInvWindow,Msg,wParam,lParam);
end;

procedure TmsEstTimer.UpdateTimer;
begin
  KillTimer(FInvWindow,1);
  if (FInterval<>0) and FEnabled then
  begin
    if SetTimer(FInvWindow,1,FInterval,nil)=0 then
      raise EOutOfResources.Create(SNoTimers);
  end;
end;

procedure TmsEstTimer.SetInterval(Value: Cardinal);
begin
  if FInterval<>Value then
  begin
    FInterval:=Value;
    UpdateTimer;
  end;
end;

procedure TmsEstTimer.SetEnabled(Value: boolean);
begin
  if Value<>FEnabled then
  begin
    FEnabled:=Value;
    if Value then
      FStartTime:=Now
    else
      FStartTime:=0;
    UpdateTimer;
  end;
end;

procedure TmsEstTimer.TimerOnTimer(Sender: TObject);
var
  EstTime,ElapsedTime: TDateTime;
  CurrentCount, TotalCount: LongInt;
begin
  ElapsedTime:=Now-FStartTime;
  if Assigned(FOnDisplayElapsedTime) then
    FOnDisplayElapsedTime(Self,ElapsedTime);
  if Assigned(FOnGetCurrentCount) then
    FOnGetCurrentCount(Self,CurrentCount)
  else
    CurrentCount:=0;
  if Assigned(FOnGetTotalCount) then
    FOnGetTotalCount(Self,TotalCount)
  else
    TotalCount:=0;
  if CurrentCount<>0 then
    EstTime:=TotalCount*ElapsedTime/CurrentCount
  else
    EstTime:=0;
  if Assigned(FOnDisplayEstTime) then
    FOnDisplayEstTime(Self,EstTime);
end;

end.
